#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# SCRIPT PRINCIPAL : astk.tcl
# arguments : -debug [n] [-p nom_profil] + ...

# $Id: astk.tcl 2302 2006-10-26 11:29:20Z courtois $

# lancement depuis une autre appli (exemple depuis Salomé) :
# set argv "--from_salome"
# set argc [llength $argv]

# on cache la fenêtre en attendant la construction
wm withdraw .

set pathref [file dirname [info script]]
# cd ../..
set root [file dirname $pathref]
set root [file dirname $root]
source [file join $root share init_share.tcl]
source [file join $pathref init.tcl]

set ashare::appli "astk"
init_gene $pathref $root
unset pathref

# pointeur off sert surtout à faire quelque chose sur "." pour qu'elle
# apparaisse avant la fenetre de log
ashare::pointeur off
ashare::init_log ".astk"

# initialisation des mots clés des différents fichiers
init_mots

# initialisation des messages erreur/info/ihm
ashare::lire_msg [file join $ashare::root share $ashare::fic_msg]

# préférences utilisateur
ashare::init_prefs
init_icon $astk::config(-1,theme)
set ashare::dbg $astk::config(-1,dbglevel)

# initialisation des noms d'onglet
def_onglet

set msg [ashare::mess "info" 1 $astk::astk_version]
puts $msg

# lecture des arguments
traite_argv

# initialisation du profil
init_profil

# initialisation des couleurs, fonts...
init_ihm

# verifications
check_conf $astk::check_quit

if { $ashare::dbg } {
   ashare::mess "info" 2 $ashare::dbg }

# splash screen
set splash ""
if { $ashare::splash_screen == 1 } {
   set splash .fen_about
   a_propos . [ashare::mess "ihm" 367]
}
if { $ashare::dbg > 0 } {
   show_fen $astk::ihm(log) force
}

# initialisation des ressources
init_env $splash

# initialisation agla
init_agla

# raffraichit données+onglet agla
raffr_agla $splash INI

# définition des unités logiques
def_ul

# onglet de demarrage
set astk::profil(onglet_actif) $astk::ihm(ong,1,1)

# initialisations finies
end_init

# fenetre principale d'ASTK
astk_princ

