#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# $Id: unites_logiques.tcl 2336 2006-11-15 13:54:27Z courtois $

# definition de la table de conversion type/UL
# pour en ajouter : rajouter un bloc (incr i...)
# extrait de struc_profil.h de asterix

# nom, numéro d'unité logique, flag par défaut (DRC),
# rep=-1: non répétable y compris entre deux listes
# rep=0 : non répétable dans une liste (étude, surcharge, tests...)
# rep=1 : répétable
# rep=2 : répétable mais une seule fois en D, une seule fois en R
# type accepté : Fichier ou Répertoire (ou les deux)
#     c'est aussi le type par défaut en résultat (donc "les deux" n'est
#     pas compatible avec Résultat).
# cpr=1 : compression autorisée

#########################################################################
proc def_ul {} {
# pour profil etude
   set var "etude"
   set i -1
#  définition des UL
   incr i
   set astk::UL($var,$i,nom) comm
   set astk::UL($var,$i,num) 1
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) mail
   set astk::UL($var,$i,num) 20
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) erre
   set astk::UL($var,$i,num) 9
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) mess
   set astk::UL($var,$i,num) 6
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) resu
   set astk::UL($var,$i,num) 8
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) base
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DRC"
   set astk::UL($var,$i,rep) 2
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) bhdf
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DRC"
   set astk::UL($var,$i,rep) 2
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) cast
   set astk::UL($var,$i,num) 37
   set astk::UL($var,$i,def) "RC"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) mast
   set astk::UL($var,$i,num) 26
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) mgib
   set astk::UL($var,$i,num) 19
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) mmed
   set astk::UL($var,$i,num) 20
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) msh
   set astk::UL($var,$i,num) 19
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) msup
   set astk::UL($var,$i,num) 19
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) datg
   set astk::UL($var,$i,num) 16
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) pos
   set astk::UL($var,$i,num) 37
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) ensi
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DR"
   set astk::UL($var,$i,rep) 2
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) dat
   set astk::UL($var,$i,num) 29
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) ps
   set astk::UL($var,$i,num) 24
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) agraf
   set astk::UL($var,$i,num) 25
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) digr
   set astk::UL($var,$i,num) 26
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) rmed
   set astk::UL($var,$i,num) 80
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) unv
   set astk::UL($var,$i,num) 30
   set astk::UL($var,$i,def) "RC"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) nom
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) -1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) para
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) repe
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DR"
   set astk::UL($var,$i,rep) 2
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 1
   incr i
   set astk::UL($var,$i,nom) libr
   set astk::UL($var,$i,num) 38
   set astk::UL($var,$i,def) ""
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 1
# pour toutes les listes
   incr i
   set astk::UL($var,$i,nom) btc
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) -1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
#  nombre d'unités logiques déclarées ici
   incr i
   set astk::UL($var,nbre) $i

####################################
# pour profil tests
   set var "tests"
   set i -1
#  définition des UL
   incr i
   set astk::UL($var,$i,nom) list
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) rep_test
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) resu_test
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 0
# pour toutes les listes
   incr i
   set astk::UL($var,$i,nom) btc
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) -1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
#  nombre d'unités logiques déclarées ici
   incr i
   set astk::UL($var,nbre) $i

####################################
# pour les sources
   set var "sources"
   set i -1
#  définition des UL
   incr i
   set astk::UL($var,$i,nom) f
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) f90
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) c
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) py
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) capy
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) cata
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) hist
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) conf
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) unig
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) datg
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) cmat
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "D"
   set astk::UL($var,$i,rep) 1
   set astk::UL($var,$i,FR)  "FR"
   set astk::UL($var,$i,cpr) 0
#  nombre d'unités logiques déclarées ici
   incr i
   set astk::UL($var,nbre) $i

####################################
# pour surcharge
   set var "surcharge"
   set i -1
#  définition des UL
   incr i
   set astk::UL($var,$i,nom) exec
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DR"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) cmde
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DR"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "R"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) ele
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "DR"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
   incr i
   set astk::UL($var,$i,nom) forlib
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) 0
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
# pour toutes les listes
   incr i
   set astk::UL($var,$i,nom) btc
   set astk::UL($var,$i,num) 0
   set astk::UL($var,$i,def) "R"
   set astk::UL($var,$i,rep) -1
   set astk::UL($var,$i,FR)  "F"
   set astk::UL($var,$i,cpr) 0
#  nombre d'unités logiques déclarées ici
   incr i
   set astk::UL($var,nbre) $i
}
